@@ -36,6 +36,7 @@ class Field:
     dataset: Optional[str] = None
     entityType: Optional[str] = None
     alias: Optional[str] = None
+    internalDataType: Optional[str] = None
 
 
 class SQLOperator(Enum):
@@ -114,7 +115,7 @@ class Condition:
         field_with_agg = self.field.column
         if self.field.aggregation is not None:
             if isinstance(self.field.aggregation, SQLFunction) or isinstance(self.field.aggregation,
-                                                                                  TimeDimensionGranularity):
+                                                                             TimeDimensionGranularity):
                 field_with_agg = self.field.aggregation.name + " ( " + field_with_agg + " )"
             else:
                 field_with_agg = self.field.aggregation + " ( " + field_with_agg + " )"
@@ -131,7 +132,7 @@ class Condition:
                 direction = self.direction
             if self.steps is not None and self.interval is not None:
                 where_condition = (
-                    field_with_agg + " " + operator + " " + self.interval + " " + direction + " " + self.steps
+                        field_with_agg + " " + operator + " " + self.interval + " " + direction + " " + self.steps
                 )
             elif self.steps is not None and self.interval is None:
                 where_condition = (
@@ -225,6 +226,7 @@ class Query:
     where: Optional[List[Union[Condition, CompositeCondition]]] = None
     orderBy: Optional[List[Sorting]] = None
     limit: Optional[Union[int, str]] = None
+    offset: Optional[Union[int, str]] = None
 
     def to_sql(self, dataset: str = None):
         sql = "SELECT {} FROM {}"
@@ -233,7 +235,8 @@ class Query:
         for field in self.fields:
             column = field.column
             if field.aggregation is not None:
-                if isinstance(field.aggregation, SQLFunction) or isinstance(field.aggregation, TimeDimensionGranularity):
+                if isinstance(field.aggregation, SQLFunction) or isinstance(field.aggregation,
+                                                                            TimeDimensionGranularity):
                     field_with_agg = field.aggregation.name + " ( " + column + " )"
                 else:
                     field_with_agg = field.aggregation + " ( " + column + " )"
@@ -286,6 +289,94 @@ class Query:
 
         return sql
 
+    def _where_spell_out(self, where):
+        conditions = []
+        for where_condition in where:
+            if isinstance(where_condition, Condition):
+                vars = []
+                if where_condition.value is not None:
+                    for var in where_condition.value:
+                        vars.append(str(var))
+                    formatted_value = "( " + ", ".join(vars) + " )"
+                else:
+                    formatted_value = None
+
+                if isinstance(where_condition.operator, str):
+                    operator = where_condition.operator
+                elif isinstance(where_condition.operator, SQLOperator) or isinstance(where_condition.operator,
+                                                                                     TimeOperator):
+                    operator = where_condition.operator.name
+                    if operator == "GOE":
+                        operator = ">="
+                    elif operator == "LOE":
+                        operator = "<="
+                    elif operator == "EQ":
+                        operator = "=="
+                    elif operator == "GT":
+                        operator = ">"
+                    elif operator == "LT":
+                        operator = "<"
+                else:
+                    operator = None
+                formatted_cond = where_condition.field.column + " (" + where_condition.field.internalDataType + ") " + operator + " " + formatted_value
+                conditions.append(formatted_cond)
+            elif isinstance(where_condition, CompositeCondition):
+                other_conds = self._where_spell_out(where_condition.conditions)
+                if where_condition.operator.name == "AND":
+                    conditions = other_conds
+                elif where_condition.operator.name == "OR":
+                    conditions.append(" OR ".join(other_conds))
+        return conditions
+
+    def spell_out(self):
+        spell_str = '# {"fields":['
+
+        fields = []
+        for field in self.fields:
+            if field.alias is None:
+                if field.aggregation is not None and isinstance(field.aggregation, str):
+                    string = '"' + field.aggregation + " " + field.column + " (" + field.internalDataType + ")" + '"'
+                elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                    string = '"' + field.aggregation.name + " " + field.column + " (" + field.internalDataType + ")" + '"'
+                else:
+                    string = '"' + field.column + " (" + field.internalDataType + ")" + '"'
+            else:
+                if field.aggregation is not None and isinstance(field.aggregation, str):
+                    string = '"' + field.aggregation + " " + field.column + " (" + field.internalDataType + ") AS " + field.alias + '"'
+                elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                    string = '"' + field.aggregation.name + " " + field.column + " (" + field.internalDataType + ") AS " + field.alias + '"'
+                else:
+                    string = '"' + field.column + " (" + field.internalDataType + ") AS " + field.alias + '"'
+            fields.append(string)
+        spell_str += ", ".join(fields) + "]"
+
+        if self.datasets is not None and self.datasets:
+            spell_str += ', "from":["{table}"]'
+
+        if self.where is not None and self.where:
+            spell_str += ', "where":['
+            condition_str = self._where_spell_out(self.where)
+            condition_str = ['"' + cond + '"' for cond in condition_str]
+            spell_str += ", ".join(condition_str) + "]"
+
+        if self.orderBy is not None and self.orderBy:
+            spell_str += ', "order_by":['
+            orderby_vec = []
+            for clause in self.orderBy:
+                orderby_vec.append('"' + clause.field + " " + clause.order.name + '"')
+            spell_str += ", ".join(orderby_vec) + "]"
+
+        if self.limit is not None:
+            spell_str += ', "limit": ' + str(self.limit)
+
+        if self.offset is not None:
+            spell_str += ', "offset": ' + str(self.offset)
+        else:
+            spell_str += ', "offset": 0'
+
+        spell_str += "}"
+        return spell_str
+
 
 @dataclass
 class SmartQuery:
@@ -293,6 +384,13 @@ class SmartQuery:
     components: Optional[List[Union[ChartComponent, MapComponent, QueryComponent, Component]]] = None
     javascript: Optional[List[str]] = None
 
+    def spell_out(self):
+        spell_array = []
+        for query in self.queries:
+            spell = query.spell_out()
+            spell_array.append(spell)
+        return spell_array
+
     @staticmethod
     def _get_tokens():
         f = open(os.path.join(os.path.dirname(__file__), "askdata_config", "compression_tokens.json"))
@@ -307,7 +405,7 @@ class SmartQuery:
             extended = token['decode']
             code = token['code']
             if extended in smartquery_json:
-                smartquery_json = re.sub(r"\b"+extended+r"\b", code, smartquery_json)
+                smartquery_json = re.sub(r"\b" + extended + r"\b", code, smartquery_json)
         return smartquery_json
 
     @staticmethod
@@ -317,5 +415,5 @@ class SmartQuery:
             code = token['code']
             decode = token['decode']
             if code in smartquery_json:
-                smartquery_json = re.sub(r"\b"+code+r"\b", decode, smartquery_json)
+                smartquery_json = re.sub(r"\b" + code + r"\b", decode, smartquery_json)
         return smartquery_json
